from argparse import ArgumentParser, RawDescriptionHelpFormatter
import base64
from pathlib import Path
import pickle
import os
import sys

import plotly.express as px

from as_analytics.lsreport import read_filetree_ls


def _get_parser():
    parser = ArgumentParser(
        description="Parser for output of `tools/lstraverse.sh`.",
        formatter_class=RawDescriptionHelpFormatter,
    )
    parser.add_argument(
        "path",
        type=str,
        help="Path of filtree as output from `tools/lstraverse.sh`.",
    )
    parser.add_argument(
        "--expid",
        type=str,
        help="Expid. Only for saving purposes",
    )
    parser.add_argument(
        "--cache",
        action="store_true",
        help="Read / write to ~/.cache"
    )
    return parser


def _get_cache_path(suffix=None) -> Path:
    key = " ".join(sys.argv) + (f" {suffix}" if suffix else "")
    encoded = base64.b64encode(key.encode("utf-8"))
    file = encoded.decode("utf-8")
    directory = os.path.basename(__file__)
    return Path.home().resolve() / ".cache" / directory / file

def _cache_read(cache, suffix=None):
    if cache is False:
        return
    cpath = _get_cache_path(suffix)
    if not cpath.exists():
        return None
    print("Reading from cache...")
    with open(cpath, "rb") as fb:
        return pickle.load(fb)

def _cache_write(cache, obj, suffix=None):
    if cache is False:
        return
    print("Writing to cache...")
    cpath = _get_cache_path(suffix)
    if not cpath.parent.exists():
        cpath.parent.mkdir(parents=True)
    with open(cpath, "wb") as fb:
        pickle.dump(obj, fb)


if __name__ == "__main__":
    args = _get_parser().parse_args()
    path = Path(args.path).resolve()
    expid = args.expid
    cache = args.cache
    prefix = f"{expid}_" if expid else ""

    df = None

    df = _cache_read(cache, "df") or read_filetree_ls(path, join=True)
    _cache_write(cache, df, "df")
    
    print("Plotting treemap.")
    fig = _cache_read(cache, "fig_treemap") or px.treemap(
        df,
        path=["Parent", "Filename"],  # Hierarchy
        values="Filesize",            # Size of each file
        hover_data=["Permissions", "Owner", "Datetime"],
    )
    _cache_write(cache, fig, "fig_treemap")
    fig.write_image(f"figures/{prefix}treemap.png")

    print("Plotting sunburst.")
    fig = _cache_read(cache, "fig_sunburst") or px.sunburst(
        df,
        path=["Parent", "Filename"],
        values="Filesize",
        hover_data=["Permissions", "Owner"]
    )
    _cache_write(cache, fig, "fig_sunburst")
    fig.write_image(f"figures/{prefix}sunburst.png")

    print("Plotting icicle.")
    fig = _cache_read(cache, "fig_icicle") or px.icicle(
        df,
        path=["Parent", "Filename"],
        values="Filesize"
    )
    _cache_write(cache, fig, "fig_icicle")
    fig.write_image(f"figures/{prefix}icicle.png")
